#region "Copyright"
/********************************************************************************
 * Copyright (c) 2007, Kerry D. Wong
 * http://www.kerrywong.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of Kerry D. Wong nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY KERRY D. WONG "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL KERRY D. WONG BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/
#endregion

#region "using"
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

using KerryWong.TiffMS.Utils;
#endregion

namespace KerryWong.TiffMS
{
    public partial class TiffMSWin : Form
    {
        public TiffMSWin()
        {
            InitializeComponent();
        }

        private void MenuItem_File_Exit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void MenuItem_File_Open_Click(object sender, EventArgs e)
        {
            if (SelectFilesDialog.ShowDialog(this) == DialogResult.OK)
            {
                foreach (string s in SelectFilesDialog.FileNames)
                {
                    FileList.Items.Add(s);
                }
            }
        }

        private void FileList_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.All;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void FileList_DragDrop(object sender, DragEventArgs e)
        {
            string draggedStr = e.Data.GetData(DataFormats.StringFormat, false) as string;

            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop, false);

            if (files != null)
            {
                foreach (string s in files)
                {
                    string ext = Path.GetExtension(s).ToUpper();

                    if (ext == ".TIF" || ext == ".TIFF")
                    {
                        FileList.Items.Add(s);
                    }
                }
            }
            else if (draggedStr != null)
            {
                int idx = FileList.IndexFromPoint(FileList.PointToClient(new Point(e.X, e.Y)));

                if (idx >= 0 && FileList.SelectedIndex >= 0)
                {
                    FileList.Items.Insert(idx, e.Data.GetData(DataFormats.Text));
                    FileList.Items.RemoveAt(FileList.SelectedIndex);
                }
            }
        }

        private void FileList_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                FileList.DoDragDrop(FileList.Text, DragDropEffects.All);
            }
            else if (e.Button == MouseButtons.Right)
            {
                PopupMenu.Show(FileList, new Point(e.X, e.Y));
            }
        }

        private void FileList_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;
        }

        private void BtnConvert_Click(object sender, EventArgs e)
        {
            List<string> files = new List<string>();

            foreach (object o in FileList.Items)
            {
                files.Add(o as string);
            }

            if (files.Count == 0) return;

            Cursor = Cursors.WaitCursor;
            try
            {

                MSUtils util = new MSUtils();

                if (RdoMultiToSingle.Checked)
                {

                    foreach (string fn in files)
                    {
                        util.ConvertToSinglePageTiffs(fn, Path.GetFileNameWithoutExtension(fn));
                    }
                }
                else
                {
                    util.ConvertToMultiPageTiff(files, OutputFileNameTextBox.Text);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                Cursor = Cursors.Default;
                MessageBox.Show("The selected files has been successfully processed.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void Rdo_CheckedChanged(object sender, EventArgs e)
        {
            if (RdoMultiToSingle.Checked)
            {
                OutputFileNameTextBox.Enabled = false;
            }
            else
            {
                OutputFileNameTextBox.Enabled = true;
            }
        }

        private void BtnClearList_Click(object sender, EventArgs e)
        {
            FileList.Items.Clear();
        }

        private void BtnOpen_Click(object sender, EventArgs e)
        {
            MenuItem_File_Open_Click(sender, e);
        }

        private void PopupMenu_ClearList_Click(object sender, EventArgs e)
        {
            BtnClearList_Click(sender, e);
        }

        private void PopupMenu_Remove_Click(object sender, EventArgs e)
        {
            if (FileList.SelectedIndices.Count > 0)
            {
                foreach (int i in FileList.SelectedIndices)
                {
                    FileList.Items.RemoveAt(i);
                }
            }
        }

        private void MenuItem_Help_About_Click(object sender, EventArgs e)
        {
            FormAbout about = new FormAbout();
            about.ShowDialog(this);
        }

        private void Menu_Help_Help_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("Readme.rtf");
        }
    }
}